import unittest

from json_fingerprint.tests.test_create import TestCreate
from json_fingerprint.tests.test_decode import TestDecode
from json_fingerprint.tests.test_find_matches import TestFindMatches
from json_fingerprint.tests.test_jfpv1 import TestJfpv1
from json_fingerprint.tests.test_match import TestMatch
from json_fingerprint.tests.test_validators import TestValidators

if __name__ == "__main__":
    unittest.main()
