class FingerprintHashFunctionError(Exception):
    pass


class FingerprintInputDataTypeError(Exception):
    pass


class FingerprintVersionError(Exception):
    pass


class FingerprintStringFormatError(Exception):
    pass


class FingerprintJSONLoadError(Exception):
    pass
